<?php
require_once 'system/setup.conf';
require_once 'warp/warplib/applet.inc';
require_once 'warp/warplib/appletutils.inc';
require_once 'warp/applets/browser/browser.inc';

class Warp_Explorer_Applet extends Warp_Browser_Applet
{

    function Warp_Explorer_Applet($caption = "Explorer", $rootpath = NULL, $mappedpath = NULL)
    {
        parent::__construct($caption, $rootpath, $mappedpath);
    }

    function getDirectory()
    {
        return "explorer";
    }

    function init(&$appMan)
    {
        parent::init($appMan);

        $this->_isFullSize = false;
        $this->_markedFile = NULL;

        $view = $appMan->queryParameter($this, "view");
        if ($view != "") {
           $this->_markedFile = &$this->_filelist[$view];
        }

        $fullname = $appMan->queryParameter($this, "full");
        if ($fullname != "") {
           $this->_markedFile = &$this->_filelist[$fullname];
           $this->_isFullSize = true;
        }
        return true;
    }

    function renderWindow(&$appMan)
    {
        if ($this->_isFullSize) {
          $filepath = $this->_markedFile->getFilePathSystem();
          $image = file_get_contents($filepath);
          $imdata = base64_encode($image);
          $fullImage = '<img src="data:image/png;base64,' . $imdata . '">';
          $html = $fullImage . "<br/><a href='javascript:window.history.back();'>Back</a>";

          return $html;

        } else {
            $top = $this->getDirectoryBrowser($appMan, $this->_dir);
            $filepane = $this->getFilePane($appMan, 0);
            $dirpane = $this->getDirectoryPane($appMan, 0);
            $preview = $this->getPreviewHTML($appMan, $this->_markedFile);
            $rhs = $appMan->getView()->split2H($filepane, $preview);
            return $top . $appMan->getView()->split2V($dirpane, $rhs);
        }
    }

    function renderInfo(&$appMan)
    {
        $p = parent::renderInfo($appMan);
        $p.= "<p>This is the Explorer variant, that also allows you to preview .txt files.";
        return $p;
    }

    function getPreviewHTML(&$appMan, $f)
    {
       if ($f) {
           # We preview text files in a separate area
           $fn = $f->getFileTitle();
           $filepath = $f->getFilePathSystem();
           if (appletUtils::isTextFile($filepath)) {
              $txt = file_get_contents($filepath);
              return "<hr><h3>$fn</h3><textarea rows=16 cols=50 readonly>$txt</textarea>";
    
           # We embed images using base64, because we can't be sure the path will be exposed through the server 
           } else if (appletUtils::isImageFile($filepath)) {
              $image = file_get_contents($filepath);
              $imdata = base64_encode($image);
              $preview = '<img width="128" src="data:image/png;base64,' . $imdata . '">';
              return "<hr>$preview";
           }
       }

       return "";
    }

    function renderFileLine($appMan, $f)
    {
       $filepath = $f->getFilePathSystem();

       if (appletUtils::isTextFile($filepath)) {
          return $appMan->getAppletLink($this, "view", $f->_idx, $f->getFileTitle())."<br/>";

       } else if (appletUtils::isImageFile($filepath)) {
          $imageLink  = $appMan->getAppletLink($this, "view", $f->_idx, $f->getFileTitle());
          $imageLink .= "&nbsp;";
          $imageLink .= $appMan->getAppletLink($this, "full", $f->_idx, '[O]')."<br/>";
          return $imageLink; 
       }
       return parent::renderFileLine($appMan, $f);
    }

}

?>


